//
//  EAGLView.m
//  es
//
//  Created by kunii on 09/05/17.
//  Copyright __MyCompanyName__ 2009. All rights reserved.
//



#import <QuartzCore/QuartzCore.h>
#import <OpenGLES/EAGLDrawable.h>

#import "EAGLView.h"

#define USE_DEPTH_BUFFER 1

// A class extension to declare private methods
@interface EAGLView ()

@property (nonatomic, retain) EAGLContext *context;
@property (nonatomic, assign) NSTimer *animationTimer;

- (BOOL) createFramebuffer;
- (void) destroyFramebuffer;

@end


@implementation EAGLView

@synthesize context;
@synthesize animationTimer;
@synthesize animationInterval;


/*
	レイヤーClassを返す事で、この返したクラスを使ってCAEAGLLayerインスタンス（UIViewは
	CALayerクラスを使うので、そのままだとCALayerインスタンスになる）が作成される。
	結果、self.layerにはCAEAGLLayerインスタンスが割り付けられる。
*/
+ (Class)layerClass {
	return [CAEAGLLayer class];
}


/*
	このEAGLViewは xibファイルから読み込まれる. その場合、読み込み直後に -initWithCoder:を
	呼び出すようになっているので、ここで、CALayerインスタンスをCAEAGLLayerインスタンスに
	変えるために必要な処理を追加する。
	
	OpenGLが高速に動作するための設定をおこなっている。
		レイヤーを不透明にする。
		表示後も内容を保持するか設定する（デフォルト値：NO）。
		色空間をRGBA順、各8ビットに設定する（デフォルト値：kEAGLColorFormatRGBA8）。
*/
- (id)initWithCoder:(NSCoder*)coder {
    if ((self = [super initWithCoder:coder])) {
        CAEAGLLayer *eaglLayer = (CAEAGLLayer *)self.layer;	//	CAEAGLLayerインスタンスを取り出す。
        eaglLayer.opaque = YES;								//	レイヤーを不透明にする。
        eaglLayer.drawableProperties = [NSDictionary dictionaryWithObjectsAndKeys:
                                        [NSNumber numberWithBool:YES], kEAGLDrawablePropertyRetainedBacking, 
										kEAGLColorFormatRGBA8, kEAGLDrawablePropertyColorFormat, nil];
        //	EAGLContextの準備
        context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];
        //	成功したか確認。
        if (!context || ![EAGLContext setCurrentContext:context]) {
            [self release];
            return nil;
        }
        //	アニメーションの間隔を秒間60コマに設定。
        animationInterval = 1.0 / 60.0;
    }
    return self;
}

static double angle = 0.0;				//	4面体の回転角

//	4面体、描画
static void drawTetra()
{
    const GLfloat tetraVertices[] = {	//	今回は4面体を定義、Z座標も当然指定。頂点も増えている。
		-0.5f, -0.5f, 0.5,
		 0.5f, -0.5f, 0.5,
		 0.0f,  0.5f, 0.0,
		 0.0f, -0.5f,-0.5,
		-0.5f, -0.5f, 0.5,
		 0.5f, -0.5f, 0.5
    };
	const GLubyte tetraColors[] = {		//	頂点も増えているんで、こっちも加増。
		255,   0,   0, 255,
		255,   0,   0, 255,
		255,   0,   0, 255,
		255,   0,   0, 255,
		255,   0,   0, 255,
		255,   0,   0, 255
	};

	glVertexPointer(3, GL_FLOAT, 0, tetraVertices);	//	X,Y,Z指定なので 3 にする。
	glEnableClientState(GL_VERTEX_ARRAY);
	glColorPointer(4, GL_UNSIGNED_BYTE, 0, tetraColors);
	glEnableClientState(GL_COLOR_ARRAY);
	
    glLoadIdentity();								//	現在のアフィン行列を、かけても何もおこらない設定する。
	glRotatef(angle, 0.0f, 1.0f, 0.0f);				//	Y軸回転（移動の後に置くのが重要）
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 6);			//	6座標を使って描画させる
}

- (void)drawView {
	[EAGLContext setCurrentContext:context];					//	カレントコンテキストを設定
    glBindFramebufferOES(GL_FRAMEBUFFER_OES, viewFramebuffer);	//	フレームバッファ設定（別のスレッド差し替えられている場合を考え、毎回設定か？）
    glViewport(0, 0, backingWidth, backingHeight);				//	viewFramebufferから取り出した画面サイズ設定。

	//	視点設定
    glMatrixMode(GL_PROJECTION);								//	これから呼び出すアフィン行列関係のAPIは、投影に関するものという宣言。
    glLoadIdentity();											//	現在のアフィン行列を、かけても何もおこらない設定する。
    glFrustumf(-1.0f, 1.0f, -1.5, 1.5, 3.0f, 100.0f);			//	透視投影。
	glRotatef(20.0f, 1.0f, 0.0f, 0.0f);
	glTranslatef(0.0f, -1.0f, -4.0f);

	//	描画領域の初期化
	glEnable(GL_DEPTH_TEST);									//	隠面消去処理を利用
    glClearColor(0.5f, 0.5f, 0.5f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);			//	隠面消去処理用バッファのクリア

	//	物体設定
    glMatrixMode(GL_MODELVIEW);
	drawTetra();
	angle += 0.3;												//	回転を加算

	//	画面への表示
	glBindRenderbufferOES(GL_RENDERBUFFER_OES, viewRenderbuffer);	
   [context presentRenderbuffer:GL_RENDERBUFFER_OES];			//	presentRenderbuffer直前にglBindRenderbufferOESとして間に合うということは...
}

/*
	レイヤー矩形が変わった
*/
- (void)layoutSubviews {
    [EAGLContext setCurrentContext:context];		//	カレントコンテキストを設定
    [self destroyFramebuffer];						//	古いフレームバッファを破棄
    [self createFramebuffer];						//	新しいフレームバッファを用意
    [self drawView];
}

/*
	新しいフレームバッファを用意する。
	関連するレンダーバッファ、ディプスバッファも用意する。
	レイヤー矩形が変わった時は、毎回呼び出す。
*/
- (BOOL)createFramebuffer {
    glGenFramebuffersOES(1, &viewFramebuffer);		//	viewFramebufferにOpenGL ESシステム側からフレーム用の固有値（名称）を設定してもらう
    glGenRenderbuffersOES(1, &viewRenderbuffer);	//	viewRenderbufferにOpenGL ESシステム側からレンダリング用の固有値（名称）を設定してもらう
    
    glBindFramebufferOES(GL_FRAMEBUFFER_OES, viewFramebuffer);		//	OpenGL ESシステム側にフレーム用として認識させる
    glBindRenderbufferOES(GL_RENDERBUFFER_OES, viewRenderbuffer);	//	OpenGL ESシステム側にレンダリング用としてバッファを作成させる
    [context renderbufferStorage:GL_RENDERBUFFER_OES fromDrawable:(CAEAGLLayer*)self.layer];	//	レイヤに割り合て
    glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_COLOR_ATTACHMENT0_OES, GL_RENDERBUFFER_OES, viewRenderbuffer);	//	色用のバッファとして
    
    glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_WIDTH_OES, &backingWidth);		//	レンダーバッファの幅を得る
    glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_HEIGHT_OES, &backingHeight);	//	高さ
    
    if (USE_DEPTH_BUFFER) {
        glGenRenderbuffersOES(1, &depthRenderbuffer);	//	depthRenderbufferにOpenGL ESシステム側からレンダリング用の固有値（名称）を設定してもらう
        glBindRenderbufferOES(GL_RENDERBUFFER_OES, depthRenderbuffer);	//	OpenGL ESシステム側にレンダリング用として認識させる
        glRenderbufferStorageOES(GL_RENDERBUFFER_OES, GL_DEPTH_COMPONENT16_OES, backingWidth, backingHeight);	//	Zバッファとして
        glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_DEPTH_ATTACHMENT_OES, GL_RENDERBUFFER_OES, depthRenderbuffer);	//	フレームにZバッファを登録
    }
    
	//	フレームバッファは正常に用意できたか確認する。
    if(glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES) != GL_FRAMEBUFFER_COMPLETE_OES) {
        NSLog(@"failed to make complete framebuffer object %x", glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES));
        return NO;
    }
     
    return YES;
}

/*
	古いフレームバッファを破棄する。
	関連するレンダーバッファ、ディプスバッファも破棄する。
*/
- (void)destroyFramebuffer {
    glDeleteFramebuffersOES(1, &viewFramebuffer);
    viewFramebuffer = 0;
    glDeleteRenderbuffersOES(1, &viewRenderbuffer);
    viewRenderbuffer = 0;
    if(depthRenderbuffer) {
        glDeleteRenderbuffersOES(1, &depthRenderbuffer);
        depthRenderbuffer = 0;
    }
}


- (void)startAnimation {
    self.animationTimer = [NSTimer scheduledTimerWithTimeInterval:animationInterval target:self selector:@selector(drawView) userInfo:nil repeats:YES];
}


- (void)stopAnimation {
    self.animationTimer = nil;
}


- (void)setAnimationTimer:(NSTimer *)newTimer {
    [animationTimer invalidate];
    animationTimer = newTimer;
}


- (void)setAnimationInterval:(NSTimeInterval)interval {
    
    animationInterval = interval;
    if (animationTimer) {
        [self stopAnimation];
        [self startAnimation];
    }
}


- (void)dealloc {
    
    [self stopAnimation];
    
    if ([EAGLContext currentContext] == context) {
        [EAGLContext setCurrentContext:nil];
    }
    
    [context release];  
    [super dealloc];
}

@end
